// src/composables/useParticles.js
import { ref, onMounted, onUnmounted } from 'vue'
import { useRafFn } from '@vueuse/core'

/**
 * 粒子系统组合式函数
 * @param {Object} options - 配置选项
 * @param {HTMLElement} options.canvas - 画布DOM元素
 * @param {number} [options.density=100] - 粒子密度
 * @param {string} [options.color='#ffffff'] - 粒子颜色
 * @param {number} [options.minRadius=1] - 最小半径
 * @param {number} [options.maxRadius=3] - 最大半径
 * @param {number} [options.maxSpeed=0.5] - 最大速度
 * @param {number} [options.lineLength=150] - 连线最大距离
 * @param {boolean} [options.interactive=true] - 是否启用交互
 * @param {number} [options.particleOpacity=0.7] - 粒子透明度
 * @param {number} [options.lineOpacity=0.3] - 连线透明度
 * @returns {Object} 控制方法
 */
export function useParticles(options) {
  const {
    canvas,
    density = 100,
    color = '#ffffff',
    minRadius = 1,
    maxRadius = 3,
    maxSpeed = 0.5,
    lineLength = 150,
    interactive = true,
    particleOpacity = 0.7,
    lineOpacity = 0.3
  } = options

  const ctx = canvas.getContext('2d')
  const particles = []
  const mouse = { x: null, y: null, radius: 100 }
  const canvasSize = ref({ width: 0, height: 0 })

  // 粒子类
  class Particle {
    constructor() {
      this.x = Math.random() * canvasSize.value.width
      this.y = Math.random() * canvasSize.value.height
      this.size = Math.random() * (maxRadius - minRadius) + minRadius
      this.density = Math.random() * 30 + 1
      this.vx = Math.random() * maxSpeed * 2 - maxSpeed
      this.vy = Math.random() * maxSpeed * 2 - maxSpeed
      this.baseX = this.x
      this.baseY = this.y
    }

    draw() {
      ctx.beginPath()
      ctx.arc(this.x, this.y, this.size, 0, Math.PI * 2)
      ctx.closePath()
      ctx.fillStyle = color
      ctx.globalAlpha = particleOpacity
      ctx.fill()
    }

    update() {
      // 边界检测
      if (this.x < 0 || this.x > canvasSize.value.width) {
        this.vx = -this.vx
      }

      if (this.y < 0 || this.y > canvasSize.value.height) {
        this.vy = -this.vy
      }

      // 鼠标交互
      if (interactive) {
        const dx = mouse.x - this.x
        const dy = mouse.y - this.y
        const distance = Math.sqrt(dx * dx + dy * dy)

        if (distance < mouse.radius) {
          const forceDirectionX = dx / distance
          const forceDirectionY = dy / distance
          const force = (mouse.radius - distance) / mouse.radius * 2
          const directionX = forceDirectionX * force * this.density
          const directionY = forceDirectionY * force * this.density

          this.x -= directionX
          this.y -= directionY
        } else {
          if (this.x !== this.baseX) {
            const dx = this.baseX - this.x
            this.x += dx / 20
          }
          if (this.y !== this.baseY) {
            const dy = this.baseY - this.y
            this.y += dy / 20
          }
        }
      }

      // 移动粒子
      this.x += this.vx
      this.y += this.vy
    }
  }

  // 初始化粒子
  const initParticles = () => {
    particles.length = 0
    const particleCount = Math.floor((canvasSize.value.width * canvasSize.value.height) / 10000 * density)

    for (let i = 0; i < particleCount; i++) {
      particles.push(new Particle())
    }
  }

  // 绘制连线
  const drawLines = () => {
    for (let a = 0; a < particles.length; a++) {
      for (let b = a; b < particles.length; b++) {
        const dx = particles[a].x - particles[b].x
        const dy = particles[a].y - particles[b].y
        const distance = Math.sqrt(dx * dx + dy * dy)

        if (distance < lineLength) {
          ctx.beginPath()
          ctx.strokeStyle = color
          ctx.globalAlpha = lineOpacity * (1 - distance / lineLength)
          ctx.lineWidth = 0.5
          ctx.moveTo(particles[a].x, particles[a].y)
          ctx.lineTo(particles[b].x, particles[b].y)
          ctx.stroke()
        }
      }
    }
  }

  // 动画循环
  const animate = () => {
    ctx.clearRect(0, 0, canvasSize.value.width, canvasSize.value.height)

    particles.forEach(particle => {
      particle.update()
      particle.draw()
    })

    drawLines()
  }

  // 处理鼠标移动
  const handleMouseMove = (event) => {
    if (!interactive) return

    const rect = canvas.getBoundingClientRect()
    mouse.x = event.clientX - rect.left
    mouse.y = event.clientY - rect.top
  }

  // 处理鼠标离开
  const handleMouseOut = () => {
    mouse.x = null
    mouse.y = null
  }

  // 处理窗口大小变化
  const handleResize = () => {
    canvasSize.value.width = canvas.offsetWidth
    canvasSize.value.height = canvas.offsetHeight
    canvas.width = canvasSize.value.width
    canvas.height = canvasSize.value.height
    initParticles()
  }

  // 启动动画
  const { pause, resume } = useRafFn(animate, { immediate: false })

  // 初始化
  onMounted(() => {
    handleResize()
    window.addEventListener('resize', handleResize)
    if (interactive) {
      canvas.addEventListener('mousemove', handleMouseMove)
      canvas.addEventListener('mouseout', handleMouseOut)
    }
    resume()
  })

  // 清理
  onUnmounted(() => {
    pause()
    window.removeEventListener('resize', handleResize)
    if (interactive) {
      canvas.removeEventListener('mousemove', handleMouseMove)
      canvas.removeEventListener('mouseout', handleMouseOut)
    }
  })

  // 暴露控制方法
  return {
    pause,
    resume,
    updateConfig(newOptions) {
      Object.assign(options, newOptions)
      initParticles()
    }
  }
}
